/**
   A BTree is defined by the following grammar.

     BTree ::=  '(' String BTree BTree ')'
             |  '(' String '()'  BTree ')'
             |  '(' String BTree  '()' ')'
             |  String

   where '(' and ')' and '()' are literals. This language also
   assumes that all tokens have white space around them.

   The static method getBTree() is essentially a recursive descent
   parser for this BTree langauge. Input strings are "tokenized" by
   the very bare bones tokenizer contained in the file Tokenizer.java.
   The requirement that all tokens in the input string be surrounded
   by white space makes the tokenizer almost trivial to write.
*/

public class BuildBTreeParseTree
{
   public static Tree buildTree(String expression)
   {
      Tokenizer tokens = new Tokenizer(expression);  // Tokenizer is defined
                                                     // in Tokenizer.java
      Tree result = getParseTree(tokens); // parse the tokens

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new Error("syntax error: unexpected input: "+tokens.peekToken()+"\n"+tokens);

      return result;
   }//buildTree()


   private static Tree getParseTree(Tokenizer tokens)
   {
      Tree result = new Tree( "Btree" );

      String token = tokens.nextToken(); // consume one token
      if ( token.equals("(") )           // look for a parenthesized tree
      {
         // add the opening "("
         result.addSubTree(new Tree(token));

         // consume the root string
         result.addSubTree(new Tree( tokens.nextToken() ));

         // consume left sub tree
         if ( tokens.peekToken().equals("()") )
         {
            result.addSubTree(new Tree( tokens.nextToken() ));
         }
         else
         {
            result.addSubTree( getParseTree(tokens) );
         }

         // consume right sub tree
         if ( tokens.peekToken().equals("()") )
         {
            result.addSubTree(new Tree( tokens.nextToken() ));
         }
         else
         {
            result.addSubTree( getParseTree(tokens) );
         }

         // consume the matching ")"
         if ( tokens.peekToken().equals(")") )
         {
            result.addSubTree(new Tree( tokens.nextToken() ));
         }
         else
         {
            throw new Error("syntax error: expected \")\"" + "\n" + tokens);
         }
      }
      else // the tree must be just the root
      {
         result.addSubTree(new Tree(token));
      }
      return result;
   }//getParseTree()
}//BuildBTreeParseTree
